@@ -3,12 +3,11 @@
  */
 package org.gcube.common.storagehubwrapper.server.converter;
 
-import java.util.ArrayList;
 import java.util.List;
-import java.util.function.Function;
 
 import org.gcube.common.storagehub.model.Metadata;
 import org.gcube.common.storagehub.model.items.AbstractFileItem;
+import org.gcube.common.storagehub.model.items.ExternalLink;
 import org.gcube.common.storagehub.model.items.ExternalURL;
 import org.gcube.common.storagehub.model.items.FolderItem;
 import org.gcube.common.storagehub.model.items.GCubeItem;
@@ -26,13 +25,13 @@ import org.gcube.common.storagehub.model.items.nodes.accounting.AccountEntry;
 import org.gcube.common.storagehub.model.service.Version;
 import org.gcube.common.storagehubwrapper.shared.tohl.WorkspaceItem;
 import org.gcube.common.storagehubwrapper.shared.tohl.WorkspaceItemType;
-import org.gcube.common.storagehubwrapper.shared.tohl.impl.AccountingEntry;
 import org.gcube.common.storagehubwrapper.shared.tohl.impl.FileItem;
 import org.gcube.common.storagehubwrapper.shared.tohl.impl.GcubeItem;
 import org.gcube.common.storagehubwrapper.shared.tohl.impl.ImageFile;
 import org.gcube.common.storagehubwrapper.shared.tohl.impl.PDFFile;
 import org.gcube.common.storagehubwrapper.shared.tohl.impl.PropertyMap;
 import org.gcube.common.storagehubwrapper.shared.tohl.impl.URLFile;
+import org.gcube.common.storagehubwrapper.shared.tohl.impl.URLFileItem;
 import org.gcube.common.storagehubwrapper.shared.tohl.impl.WorkspaceFileVersion;
 import org.gcube.common.storagehubwrapper.shared.tohl.impl.WorkspaceFolder;
 import org.gcube.common.storagehubwrapper.shared.tohl.impl.WorkspaceSharedFolder;
@@ -51,18 +50,29 @@ public class HLMapper {
 
 	private static Logger logger = LoggerFactory.getLogger(HLMapper.class);
 
-	public static Function<org.gcube.common.storagehub.model.items.nodes.accounting.AccountEntry, AccountingEntry> toAccountingEntry = new Function<org.gcube.common.storagehub.model.items.nodes.accounting.AccountEntry, AccountingEntry>() {
+//	public static Function<org.gcube.common.storagehub.model.items.nodes.accounting.AccountEntry, AccountingEntry> toAccountingEntry = new Function<org.gcube.common.storagehub.model.items.nodes.accounting.AccountEntry, AccountingEntry>() {
+//
+//	    public AccountingEntry apply(org.gcube.common.storagehub.model.items.nodes.accounting.AccountEntry accountingEntry) {
+//
+//	    	return new AccountingEntry(
+//	    		accountingEntry.getUser(),
+//	    		accountingEntry.getDate(),
+//	    		accountingEntry.getType(),
+//	    		accountingEntry.getVersion(),
+//	    		accountingEntry.getPrimaryType());
+//	    }
+//	}
 
-	    public AccountingEntry apply(org.gcube.common.storagehub.model.items.nodes.accounting.AccountEntry accountingEntry) {
-
-	    	return new AccountingEntry(
-	    		accountingEntry.getUser(),
-	    		accountingEntry.getDate(),
-	    		accountingEntry.getType().name(),
-	    		accountingEntry.getVersion(),
-	    		accountingEntry.getPrimaryType());
-	    }
-	};
+	/**
+	 * To URL file.
+	 *
+	 * @param extLink the ext link
+	 * @return the URL file
+	 * @throws Exception the exception
+	 */
+	public static URLFile toURLFile(ExternalLink extLink) throws Exception {
+		return (URLFile) toWorkspaceItem(extLink, false, false, false);
+	}
 
 
 	/**
@@ -71,6 +81,7 @@ public class HLMapper {
 	 * @param <T> the generic type
 	 * @param item the item
 	 * @return the t
+	 * @throws Exception the exception
 	 */
 	public static<T extends WorkspaceItem> T toWorkspaceItem(Item item) throws Exception{
 
@@ -78,22 +89,23 @@ public class HLMapper {
 	}
 
 
+
 	/**
 	 * To accounting entries.
 	 *
 	 * @param item the item
 	 * @return the list
 	 */
-	public static List<org.gcube.common.storagehubwrapper.shared.tohl.AccountingEntry> toAccountingEntries(Item item){
+	public static List<AccountEntry> toAccountingEntries(Item item){
 
-    	List<org.gcube.common.storagehubwrapper.shared.tohl.AccountingEntry> accountingEntries = null;
+    	List<AccountEntry> accountingEntries = null;
 
 		Accounting accounting = item.getAccounting();
-    	if(accounting!=null && accounting.getEntries().size()>0){
-    		accountingEntries = new ArrayList<org.gcube.common.storagehubwrapper.shared.tohl.AccountingEntry>(accounting.getEntries().size());
-    		for (AccountEntry ae : accounting.getEntries()) {
-    			accountingEntries.add(toAccountingEntry.apply(ae));
-			}
+    	if(accounting!=null){
+    		accountingEntries = accounting.getEntries();
+//    		for (AccountEntry ae : accounting.getEntries()) {
+//    			accountingEntries.add(toAccountingEntry.apply(ae));
+//			}
     	}
 
     	return accountingEntries;
@@ -111,7 +123,7 @@ public class HLMapper {
 
     	PropertyMap pm = null;
 		if(metadata!=null)
-			pm = new PropertyMap(metadata.getValues());
+			pm = new PropertyMap(metadata.getMap());
 
 		//IN CASE OF EMPTY MAP RETURNS NULL
 		if(pm!=null && pm.getValues()!=null && pm.getValues().isEmpty())
@@ -146,11 +158,12 @@ public class HLMapper {
 	 * @param withFileDetails the with file details
 	 * @param withMapProperties the with map properties
 	 * @return the t
+	 * @throws Exception the exception
 	 */
 	public static<T extends WorkspaceItem> T toWorkspaceItem(Item item, boolean withAccounting, boolean withFileDetails, boolean withMapProperties) throws Exception{
 
 		try{
-	    	List<org.gcube.common.storagehubwrapper.shared.tohl.AccountingEntry> accountingEntries = null;
+	    	List<AccountEntry> accountingEntries = null;
 
 			if(withAccounting)
 				accountingEntries = toAccountingEntries(item);
@@ -162,6 +175,7 @@ public class HLMapper {
 	    	String itemName = item.getName();
 	    	boolean isRoot = false;
 
+			logger.trace("The item: "+item);
 	    	logger.trace("Converting Item: "+item.getName() + " with id: "+item.getId());
 
 	    	//THE ITEM IS A KIND OF FOLDER
@@ -198,7 +212,7 @@ public class HLMapper {
 //	    				for (String key : map.keySet()) {
 //	    					logger.debug("Key: "+key+ "value: "+map.get(key));
 //						}
-	    				String isPublic = (String) item.getPropertyMap().getValues().get("isPublic");
+	    				String isPublic = (String) item.getMetadata().getMap().get("isPublic");
 	    				logger.debug("The item name: "+item.getName()+ " has isPublic property like: "+isPublic);
 	    				isPublicItem = isPublic!=null?isPublic.toLowerCase().contains("true"):false;
 	    			}catch(Exception e){
@@ -216,8 +230,7 @@ public class HLMapper {
 	    		type = WorkspaceItemType.FILE_ITEM;
 	    		FileItemType fileItemType = null;
 	    		if(item instanceof ExternalURL){
-	    			ExternalURL eURL = (ExternalURL) item; //??
-	    			theItem = new URLFile();
+	    			theItem = new URLFileItem();
 	    			fileItemType = FileItemType.URL_DOCUMENT;
 	    		}else if(item instanceof GenericFileItem){
 	    			GenericFileItem gFI = (GenericFileItem) item; //??
@@ -264,12 +277,22 @@ public class HLMapper {
 	    		//TODO ADD VERSION NAME
 
 	    	}
-
+	    	
 	    	//ADDING (GCUBE) PROPERTIES
 	    	PropertyMap pm = null;
 	    	if(withMapProperties || isFolder){
 	    		//System.out.println("Setting map: "+item.getPropertyMap().getValues());
-	    		pm = toPropertyMap(item.getPropertyMap());
+	    		pm = toPropertyMap(item.getMetadata());
+	    	}
+	    	
+	    	//CONVERTING EXTERNALLINK
+	    	if(item instanceof ExternalLink) {
+    			logger.trace("FOUND an ExternalLink...");
+	    		type = WorkspaceItemType.URL_ITEM;
+    			ExternalLink extLink =  (ExternalLink) item;
+    			theItem = new URLFile();
+    			URLFile theURLFile = (URLFile) theItem;
+    			theURLFile.setValue(extLink.getValue());
 	    	}
 
 	    	//CONVERTING TRASH ITEM
@@ -309,9 +332,6 @@ public class HLMapper {
 				logger.debug("Wrapped as GcubeItem");
 			}
 
-			logger.trace("The item: "+item);
-			logger.trace("The item id: "+item.getId());
-
 			if(theItem==null){
 				logger.info("Mapping unknown object as simple FileItem");
 				theItem = new FileItem();
@@ -337,10 +357,11 @@ public class HLMapper {
 			theItem.setAccounting(accountingEntries);
 			theItem.setType(type);
 			theItem.setFolder(isFolder);
-			theItem.setRoot(isRoot); //IS ALWAYS FALSE
+			theItem.setRoot(isRoot); //THIS IS ALWAYS FALSE. SHUB DOES NOT RETURN THIS INFORMATION
 			theItem.setPropertyMap(pm);
 
 			logger.debug("Wrapped WsItem: "+theItem);
+			logger.debug("Wrapped WsItem has type: "+theItem.getClass().getName());
 
 			return (T) theItem;
 
@@ -352,4 +373,7 @@ public class HLMapper {
 
 	}
 
+
+
+	
 }
